@@ -1,10 +1,10 @@
 import asyncio
 import functools
 from memorix_client_redis.features.api.hash_key import hash_key
-from memorix_client_redis.features.api.json import from_json, to_json
+from memorix_client_redis.features.api.json import from_json, to_json, bytes_to_str
 from typing import Generic, Optional, Type, TypeVar, cast
 from ..api import Api, ApiDefaults
-from .cache_options import CacheSetOptions, CacheSetOptionsExpire
+from .cache_options import CacheSetOptions
 
 KT = TypeVar("KT")
 PT = TypeVar("PT")
@@ -16,17 +16,20 @@ class CacheItem(Generic[KT, PT]):
         api: Api,
         id: str,
         payload_class: Type[PT],
+        options: Optional[CacheSetOptions] = None,
     ) -> None:
         self._api = api
         self._id = id
         self._payload_class = payload_class
+        self._options = options
 
     def get(self, key: KT) -> Optional[PT]:
-        res = self._api._redis.get(hash_key(self._id, key=key))
-        if res is None:
+        data_bytes = self._api._redis.get(hash_key(self._id, key=key))
+        if data_bytes is None:
             return None
 
-        payload = from_json(value=res, data_class=self._payload_class)
+        data_str = bytes_to_str(data_bytes)
+        payload = from_json(value=data_str, data_class=self._payload_class)
         return payload
 
     async def async_get(self, key: KT) -> Optional[PT]:
@@ -47,24 +50,33 @@ class CacheItem(Generic[KT, PT]):
         payload: PT,
         options: Optional[CacheSetOptions] = None,
     ) -> Optional[bool]:
-        expire: Optional[CacheSetOptionsExpire] = None
+        merged_options = self._options
         try:
-            expire = cast(CacheSetOptions, options).expire
+            merged_options = CacheSetOptions.merge(
+                cast(ApiDefaults, self._api._defaults).cache_set_options,
+                self._options,
+            )
         except AttributeError:
-            try:
-                expire = cast(
-                    CacheSetOptions,
-                    cast(ApiDefaults, self._api._defaults).cache_set_options,
-                ).expire
-            except AttributeError:
-                pass
+            pass
+        merged_options = CacheSetOptions.merge(
+            merged_options,
+            options,
+        )
 
         payload_json = to_json(payload)
         return self._api._redis.set(
             hash_key(self._id, key=key),
             payload_json,
-            ex=expire.value if expire is not None and not expire.is_in_ms else None,
-            px=expire.value if expire is not None and expire.is_in_ms else None,
+            ex=merged_options.expire.value
+            if merged_options is not None
+            and merged_options.expire is not None
+            and not merged_options.expire.is_in_ms
+            else None,
+            px=merged_options.expire.value
+            if merged_options is not None
+            and merged_options.expire is not None
+            and merged_options.expire.is_in_ms
+            else None,
         )
 
     async def async_set(
@@ -98,9 +110,22 @@ class CacheItemNoKey(CacheItem[None, PT]):
         return await CacheItem.async_get(casted_self, key=None)
 
     # Different signature on purpose
-    def set(self, payload: PT) -> Optional[bool]:  # type: ignore
-        return CacheItem.set(self, key=None, payload=payload)
+    def set(  # type: ignore
+        self,
+        payload: PT,
+        options: Optional[CacheSetOptions] = None,
+    ) -> Optional[bool]:
+        return CacheItem.set(self, key=None, payload=payload, options=options)
 
     # Different signature on purpose
-    async def async_set(self, payload: PT) -> Optional[bool]:  # type: ignore
-        return await CacheItem.async_set(self, key=None, payload=payload)
+    async def async_set(  # type: ignore
+        self,
+        payload: PT,
+        options: Optional[CacheSetOptions] = None,
+    ) -> Optional[bool]:
+        return await CacheItem.async_set(
+            self,
+            key=None,
+            payload=payload,
+            options=options,
+        )
